package ru.dbotthepony.kommons.util

import java.util.function.Supplier


fun <T> Comparator<T>.nullsFirst(): Comparator<T> {
	return Comparator.nullsFirst(this)
}

fun <T> Comparator<T>.nullsLast(): Comparator<T> {
	return Comparator.nullsLast(this)
}

class MappedComparator<T, O>(private val parent: Comparator<O>, private val mapper: (T) -> O) : Comparator<T> {
	override fun compare(o1: T, o2: T): Int {
		return parent.compare(mapper.invoke(o1), mapper.invoke(o2))
	}

	override fun equals(other: Any?): Boolean {
		return other is MappedComparator<*, *> && parent == other.parent
	}

	override fun hashCode(): Int {
		return parent.hashCode()
	}

	override fun toString(): String {
		return "MappedComparator[$parent]"
	}
}

fun <A, B> Comparator<A>.map(mapper: (B) -> A): Comparator<B> {
	return MappedComparator(this, mapper)
}

fun <T> Comparator<T>.suppliers(): Comparator<Supplier<T>> {
	return MappedComparator(this) { it.get() }
}
