package ru.dbotthepony.kommons.test

import it.unimi.dsi.fastutil.io.FastByteArrayInputStream
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.kommons.io.DelegateSyncher
import ru.dbotthepony.kommons.util.Delegate
import kotlin.test.assertEquals

object DelegateSyncherTests {
	@Test
	@DisplayName("Delegate Syncher basic test")
	fun test() {
		val a = DelegateSyncher()
		val b = DelegateSyncher()

		a.int(0).delegate.accept(4)
		val remote = a.Remote()
		val payload = remote.write()!!

		val v = b.int(0)
		b.read(FastByteArrayInputStream(payload.array, 0, payload.length))

		assertEquals(4, v.delegate.get())
	}

	@Test
	@DisplayName("Delegate Syncher multiple delegates")
	fun multiple() {
		val a = DelegateSyncher()
		val b = DelegateSyncher()
		val ints = ArrayList<Delegate<Int>>()
		val longs = ArrayList<Delegate<Long>>()

		for (i in 0 .. 10) {
			a.int().delegate.accept(i)
			a.long().delegate.accept(i.toLong())

			ints.add(b.int())
			longs.add(b.long())
		}

		val remote = a.Remote()
		val payload = remote.write()!!

		b.read(FastByteArrayInputStream(payload.array, 0, payload.length))

		for (i in 0 .. 10) {
			assertEquals(i, ints[i].get())
			assertEquals(i.toLong(), longs[i].get())
		}
	}

	@Test
	@DisplayName("Delegate Syncher network half of delegates")
	fun half() {
		val a = DelegateSyncher()
		val b = DelegateSyncher()
		val ints = ArrayList<Delegate<Int>>()
		val longs = ArrayList<Delegate<Long>>()

		for (i in 0 .. 10) {
			if (i % 2 == 0) {
				a.int()
				a.long()
			} else {
				a.int().delegate.accept(i)
				a.long().delegate.accept(i.toLong())
			}

			ints.add(b.int())
			longs.add(b.long())
		}

		val remote = a.Remote()
		val payload = remote.write()!!

		b.read(FastByteArrayInputStream(payload.array, 0, payload.length))

		for (i in 0 .. 10) {
			if (i % 2 == 0) {
				assertEquals(0, ints[i].get())
				assertEquals(0L, longs[i].get())
			} else {
				assertEquals(i, ints[i].get())
				assertEquals(i.toLong(), longs[i].get())
			}
		}
	}

	@Test
	@DisplayName("Delegate Syncher >127 delegates")
	fun moreThanByte() {
		val a = DelegateSyncher()
		val b = DelegateSyncher()
		val ints = ArrayList<Delegate<Int>>()
		val longs = ArrayList<Delegate<Long>>()

		for (i in 0 .. 400) {
			a.int().delegate.accept(i)
			a.long().delegate.accept(i.toLong())

			ints.add(b.int())
			longs.add(b.long())
		}

		val remote = a.Remote()
		val payload = remote.write()!!

		b.read(FastByteArrayInputStream(payload.array, 0, payload.length))

		for (i in 0 .. 400) {
			assertEquals(i, ints[i].get())
			assertEquals(i.toLong(), longs[i].get())
		}
	}
}